"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const lodash = require("lodash");
const cli_utils_1 = require("@ionic/cli-utils");
const command_1 = require("@ionic/cli-utils/lib/command");
const serve_1 = require("@ionic/cli-utils/lib/serve");
const errors_1 = require("@ionic/cli-utils/lib/errors");
class ServeCommand extends command_1.Command {
    getRunner() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.runner) {
                this.runner = yield serve_1.ServeRunner.createFromProjectType(this.env, this.env.project.type);
            }
            return this.runner;
        });
    }
    getMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const options = [
                ...serve_1.COMMON_SERVE_COMMAND_OPTIONS,
                {
                    name: 'lab-host',
                    description: 'Use specific address for Ionic Lab server',
                    default: 'localhost',
                    groups: [cli_utils_1.OptionGroup.Advanced],
                },
                {
                    name: 'lab-port',
                    description: 'Use specific port for Ionic Lab server',
                    default: serve_1.DEFAULT_LAB_PORT.toString(),
                    groups: [cli_utils_1.OptionGroup.Advanced],
                },
                {
                    name: 'devapp',
                    description: 'Do not publish DevApp service',
                    type: Boolean,
                    default: true,
                    groups: [cli_utils_1.OptionGroup.Advanced],
                },
                {
                    name: 'open',
                    description: 'Do not open a browser window',
                    type: Boolean,
                    default: true,
                },
                {
                    name: 'local',
                    description: 'Disable external network usage',
                    type: Boolean,
                },
                {
                    name: 'browser',
                    description: `Specifies the browser to use (${serve_1.BROWSERS.map(b => chalk_1.default.green(b)).join(', ')})`,
                    aliases: ['w'],
                    groups: [cli_utils_1.OptionGroup.Advanced],
                },
                {
                    name: 'browseroption',
                    description: `Specifies a path to open to (${chalk_1.default.green('/#/tab/dash')})`,
                    aliases: ['o'],
                    groups: [cli_utils_1.OptionGroup.Advanced],
                },
                {
                    name: 'lab',
                    description: 'Test your apps on multiple platform types in the browser',
                    type: Boolean,
                    aliases: ['l'],
                },
                {
                    name: 'platform',
                    description: `Start serve with a specific platform (${['android', 'ios'].map(t => chalk_1.default.green(t)).join(', ')})`,
                    aliases: ['t'],
                },
                {
                    name: 'auth',
                    description: 'HTTP Basic Auth password to secure the server on your local network',
                    type: String,
                    groups: [cli_utils_1.OptionGroup.Hidden],
                },
            ];
            const exampleCommands = ['', '-c', '--local', '--lab'];
            let longDescription = `
Easily spin up a development server which launches in your browser. It watches for changes in your source files and automatically reloads with the updated build.

By default, ${chalk_1.default.green('ionic serve')} boots up a development server on all network interfaces and prints the external address(es) on which your app is being served. It also broadcasts your app to the Ionic DevApp on your network. To disable the DevApp and bind to ${chalk_1.default.green('localhost')}, use ${chalk_1.default.green('--local')}.

Try the ${chalk_1.default.green('--lab')} option to see multiple platforms at once.`;
            try {
                const runner = yield this.getRunner();
                const libmetadata = yield runner.getCommandMetadata();
                options.push(...libmetadata.options || []);
                longDescription += `\n\n${(libmetadata.longDescription || '').trim()}`;
                exampleCommands.push(...libmetadata.exampleCommands || []);
            }
            catch (e) {
                if (!(e instanceof errors_1.RunnerNotFoundException)) {
                    throw e;
                }
            }
            return {
                name: 'serve',
                type: 'project',
                description: 'Start a local dev server for app dev/testing',
                longDescription,
                exampleCommands,
                options,
            };
        });
    }
    preRun(inputs, options, runinfo) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [alias] = lodash.last(runinfo.location.path) || [undefined];
            if (alias === 'lab') {
                options['lab'] = true;
            }
            if (options['nolivereload']) {
                this.env.log.warn(`The ${chalk_1.default.green('--nolivereload')} option has been deprecated. Please use ${chalk_1.default.green('--no-livereload')}.`);
                options['livereload'] = false;
            }
            if (options['nobrowser']) {
                this.env.log.warn(`The ${chalk_1.default.green('--nobrowser')} option has been deprecated. Please use ${chalk_1.default.green('--no-open')}.`);
                options['open'] = false;
            }
            if (options['b']) {
                options['open'] = false;
            }
            if (options['noproxy']) {
                this.env.log.warn(`The ${chalk_1.default.green('--noproxy')} option has been deprecated. Please use ${chalk_1.default.green('--no-proxy')}.`);
                options['proxy'] = false;
            }
            if (options['x']) {
                options['proxy'] = false;
            }
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { serve } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/serve'));
            yield serve(this.env, inputs, options);
        });
    }
}
exports.ServeCommand = ServeCommand;
class LabCommand extends ServeCommand {
    getMetadata() {
        const _super = name => super[name];
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const metadata = yield _super("getMetadata").call(this);
            const groups = [...metadata.groups || [], cli_utils_1.CommandGroup.Hidden];
            const exampleCommands = [...metadata.exampleCommands || []].filter(c => !c.includes('--lab'));
            return Object.assign({}, metadata, { description: 'Start Ionic Lab for multi-platform dev/testing', longDescription: `
Start an instance of ${chalk_1.default.bold('Ionic Lab')}, a tool for developing Ionic apps for multiple platforms at once side-by-side.

${chalk_1.default.green('ionic lab')} is just a convenient shortcut for ${chalk_1.default.green('ionic serve --lab')}.`, groups,
                exampleCommands });
        });
    }
}
exports.LabCommand = LabCommand;
