"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path = require("path");
const chalk_1 = require("chalk");
const Debug = require("debug");
const lodash = require("lodash");
const cli_framework_1 = require("@ionic/cli-framework");
const format_1 = require("@ionic/cli-framework/utils/format");
const fs_1 = require("@ionic/cli-framework/utils/fs");
const string_1 = require("@ionic/cli-framework/utils/string");
const cli_utils_1 = require("@ionic/cli-utils");
const command_1 = require("@ionic/cli-utils/lib/command");
const errors_1 = require("@ionic/cli-utils/lib/errors");
const project_1 = require("@ionic/cli-utils/lib/project");
const emoji_1 = require("@ionic/cli-utils/lib/utils/emoji");
const debug = Debug('ionic:cli:commands:start');
class StartCommand extends command_1.Command {
    getMetadata() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { STARTER_TEMPLATES } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/start'));
            return {
                name: 'start',
                type: 'global',
                description: 'Create a new project',
                longDescription: `
This command creates a working Ionic app. It installs dependencies for you and sets up your project.

${chalk_1.default.green('ionic start')} will create a new app from ${chalk_1.default.green('template')}. You can list all templates with the ${chalk_1.default.green('--list')} option. For more information on starter templates, see the CLI documentation${chalk_1.default.cyan('[1]')}.

You can also specify a git repository URL for ${chalk_1.default.green('template')} and the existing project will be cloned.

${chalk_1.default.cyan('[1]')}: ${chalk_1.default.bold('https://ionicframework.com/docs/cli/starters.html')}
      `,
                exampleCommands: [
                    '',
                    '--list',
                    'myApp',
                    'myApp blank',
                    'myApp tabs --cordova',
                    'myApp blank --type=ionic1',
                    'myApp super --type=ionic-angular',
                    'myConferenceApp https://github.com/ionic-team/ionic-conference-app',
                ],
                inputs: [
                    {
                        name: 'name',
                        description: 'The name of your project directory',
                        validators: [cli_framework_1.validators.required],
                    },
                    {
                        name: 'template',
                        description: `The starter template to use (e.g. ${['blank', 'tabs'].map(t => chalk_1.default.green(t)).join(', ')}; use ${chalk_1.default.green('--list')} to see all)`,
                        validators: [cli_framework_1.validators.required],
                    },
                ],
                options: [
                    {
                        name: 'list',
                        description: 'List starter templates available',
                        type: Boolean,
                        aliases: ['l'],
                    },
                    {
                        name: 'type',
                        description: `Type of project to start (e.g. ${lodash.uniq(STARTER_TEMPLATES.map(t => t.type)).map(type => chalk_1.default.green(type)).join(', ')})`,
                        type: String,
                    },
                    {
                        name: 'display-name',
                        description: 'Human-friendly name (use quotes around the name)',
                        type: String,
                        aliases: ['n'],
                    },
                    {
                        name: 'cordova',
                        description: 'Include Cordova integration',
                        type: Boolean,
                    },
                    {
                        name: 'deps',
                        description: 'Do not install npm/yarn dependencies',
                        type: Boolean,
                        default: true,
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                    {
                        name: 'git',
                        description: 'Do not initialize a git repo',
                        type: Boolean,
                        default: true,
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                    {
                        name: 'link',
                        description: 'Do not ask to connect the app with the Ionic Dashboard',
                        type: Boolean,
                        default: true,
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                    {
                        name: 'pro-id',
                        description: 'Specify an app ID from the Ionic Dashboard to link',
                    },
                    {
                        name: 'bundle-id',
                        description: 'Specify the bundle ID/application ID for your app (reverse-DNS notation)',
                        groups: [cli_utils_1.OptionGroup.Advanced],
                    },
                    {
                        name: 'tag',
                        description: `Specify a tag to use for the starters (e.g. ${['latest', 'testing', 'next'].map(t => chalk_1.default.green(t)).join(', ')})`,
                        default: 'latest',
                        groups: [cli_utils_1.OptionGroup.Hidden],
                    },
                ],
            };
        });
    }
    preRun(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { STARTER_TEMPLATES } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/start'));
            const { promptToLogin } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/session'));
            // If the action is list then lets just end here.
            if (options['list']) {
                const columnHeaders = ['name', 'project type', 'description'];
                this.env.log.rawmsg(format_1.columnar(STARTER_TEMPLATES.map(({ name, type, description }) => [chalk_1.default.green(name), chalk_1.default.bold(type), description || '']), { columnHeaders }));
                throw new errors_1.FatalException('', 0);
            }
            if (options['skip-deps']) {
                this.env.log.warn(`The ${chalk_1.default.green('--skip-deps')} option has been deprecated. Please use ${chalk_1.default.green('--no-deps')}.`);
                options['deps'] = false;
            }
            if (options['skip-link']) {
                this.env.log.warn(`The ${chalk_1.default.green('--skip-link')} option has been deprecated. Please use ${chalk_1.default.green('--no-link')}.`);
                options['link'] = false;
            }
            if (options['pro-id']) {
                if (options['link'] === false) {
                    this.env.log.warn(`The ${chalk_1.default.green('--no-link')} option has no effect with ${chalk_1.default.green('--pro-id')}.`);
                }
                options['link'] = true;
            }
            const proAppId = options['pro-id'] ? String(options['pro-id']) : undefined;
            if (this.env.project.directory) {
                const confirm = yield this.env.prompt({
                    type: 'confirm',
                    name: 'confirm',
                    message: 'You are already in an Ionic project directory. Do you really want to start another project here?',
                    default: false,
                });
                if (!confirm) {
                    this.env.log.msg('Not starting project within existing project.');
                    throw new errors_1.FatalException();
                }
            }
            if (options['v1'] || options['v2']) {
                const type = options['v1'] ? 'ionic1' : 'ionic-angular';
                throw new errors_1.FatalException(`Sorry! The ${chalk_1.default.green('--v1')} and ${chalk_1.default.green('--v2')} flags have been removed.\n` +
                    `Use the ${chalk_1.default.green('--type')} option. (${chalk_1.default.green('ionic start --help')})\n\n` +
                    `For ${chalk_1.default.bold(project_1.prettyProjectName(type))} projects, try ${chalk_1.default.green('ionic start ' + (inputs.length > 0 ? inputs.join(' ') + ' ' : '') + '--type=' + type)}`);
            }
            if (options['app-name']) {
                this.env.log.warn(`The ${chalk_1.default.green('--app-name')} option has been deprecated, please use ${chalk_1.default.green('--display-name')}.`);
                options['display-name'] = options['app-name'];
            }
            if (options['bundle-id']) {
                this.env.log.msg(`${chalk_1.default.green('--bundle-id')} detected, using ${chalk_1.default.green('--cordova')}`);
                options['cordova'] = true;
            }
            if (proAppId) {
                if (!(yield this.env.session.isLoggedIn())) {
                    yield promptToLogin(this.env);
                }
            }
            if (!inputs[0]) {
                if (proAppId) {
                    const { App } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/app'));
                    const token = yield this.env.session.getUserToken();
                    const appLoader = new App(token, this.env.client);
                    const app = yield appLoader.load(proAppId);
                    this.env.log.msg(`Using ${chalk_1.default.bold(app.slug)} for ${chalk_1.default.green('name')}.`);
                    inputs[0] = app.slug;
                }
                else {
                    this.env.log.info(`Every great app needs a name! ${emoji_1.emoji('😍', '')}\n` +
                        `We will use this name for the project's folder name and package name. You can change this at any time. To bypass this prompt next time, supply ${chalk_1.default.green('name')}, the first argument to ${chalk_1.default.green('ionic start')}.`);
                    const name = yield this.env.prompt({
                        type: 'input',
                        name: 'name',
                        message: 'Project name:',
                        validate: v => cli_framework_1.validators.required(v),
                    });
                    inputs[0] = name;
                }
            }
            const projectDir = path.resolve(inputs[0]);
            yield this.checkForExisting(inputs[0], projectDir);
            const clonedApp = string_1.isValidURL(inputs[1]);
            if (!clonedApp && !options['type']) {
                const recommendedType = 'angular';
                this.env.log.info(`What type of project would you like to create?\n` +
                    `We recommend ${chalk_1.default.green(recommendedType)}. To learn more about project types, see the CLI documentation${chalk_1.default.cyan('[1]')}. To bypass this prompt next time, supply the ${chalk_1.default.green('--type')} option.\n\n` +
                    `${chalk_1.default.cyan('[1]')}: ${chalk_1.default.bold('https://ionicframework.com/docs/cli/starters.html')}`);
                const type = yield this.env.prompt({
                    type: 'list',
                    name: 'template',
                    message: 'Project type:',
                    choices: () => {
                        const projectTypes = lodash.uniq(STARTER_TEMPLATES.map(t => t.type));
                        const cols = format_1.columnar(projectTypes.map(type => [`${chalk_1.default.green(type)}${type === recommendedType ? ' (recommended)' : ''}`, project_1.prettyProjectName(type)])).split('\n');
                        return cols.map((col, i) => ({
                            name: col,
                            short: projectTypes[i],
                            value: projectTypes[i],
                        }));
                    },
                });
                options['type'] = type;
            }
            if (!inputs[1]) {
                this.env.log.info(`Let's pick the perfect starter template! ${emoji_1.emoji('💪', '')}\n` +
                    `Starter templates are ready-to-go Ionic apps that come packed with everything you need to build your app. To bypass this prompt next time, supply ${chalk_1.default.green('template')}, the second argument to ${chalk_1.default.green('ionic start')}.`);
                const template = yield this.env.prompt({
                    type: 'list',
                    name: 'template',
                    message: 'Starter template:',
                    choices: () => {
                        const starterTemplates = STARTER_TEMPLATES.filter(st => st.type === options['type']);
                        const cols = format_1.columnar(starterTemplates.map(({ name, type, description }) => [chalk_1.default.green(name), chalk_1.default.bold(type), description || ''])).split('\n');
                        return cols.map((col, i) => {
                            return {
                                name: col,
                                short: starterTemplates[i].name,
                                value: starterTemplates[i].name,
                            };
                        });
                    },
                });
                inputs[1] = template;
            }
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { pkgManagerArgs } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/utils/npm'));
            const { getIonicDevAppText, getIonicProText } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/start'));
            const config = yield this.env.config.load();
            const { npmClient } = config;
            const [name, template] = inputs;
            const displayName = options['display-name'] ? String(options['display-name']) : name;
            const proAppId = options['pro-id'] ? String(options['pro-id']) : undefined;
            const bundleId = options['bundle-id'] ? String(options['bundle-id']) : undefined;
            const tag = options['tag'] ? String(options['tag']) : 'latest';
            const clonedApp = string_1.isValidURL(template);
            let linkConfirmed = typeof proAppId === 'string';
            const gitIntegration = options['git'] ? yield this.isGitSetup() : false;
            if (proAppId && !gitIntegration) {
                throw new errors_1.FatalException(`Git CLI not found on your PATH. It must be installed to connect this app to Ionic.\n` +
                    `See installation docs for git: ${chalk_1.default.bold('https://git-scm.com/book/en/v2/Getting-Started-Installing-Git')}`);
            }
            const projectDir = path.resolve(name);
            yield this.validateName(name);
            this.env.tasks.next(`Preparing directory ${chalk_1.default.green(format_1.prettyPath(projectDir))}`);
            if (this.canRemoveExisting) {
                yield fs_1.removeDirectory(projectDir);
            }
            yield fs_1.fsMkdir(projectDir, 0o777);
            if (clonedApp) {
                yield this.env.shell.run('git', ['clone', template, name, '--progress'], {});
            }
            else {
                const starterTemplate = yield this.findStarterTemplate(template, String(options['type']), tag);
                yield this.downloadStarterTemplate(projectDir, starterTemplate);
            }
            // start is weird, once the project directory is created, it becomes a
            // "project" command and so we replace the `Project` instance that was
            // autogenerated when the CLI booted up. This has worked thus far?
            this.env.project = yield cli_utils_1.getProject(projectDir, this.env);
            const shellOptions = { cwd: projectDir, stdio: ['inherit', 'ignore', 'ignore'] };
            if (!clonedApp) {
                if (!options['cordova']) {
                    const confirm = yield this.env.prompt({
                        type: 'confirm',
                        name: 'confirm',
                        message: 'Would you like to integrate your new app with Cordova to target native iOS and Android?',
                        default: false,
                    });
                    if (confirm) {
                        options['cordova'] = true;
                    }
                }
                if (options['cordova']) {
                    yield this.env.runCommand(['integrations', 'enable', 'cordova', '--quiet']);
                }
                yield this.env.project.personalize({ appName: name, bundleId, displayName });
                this.env.log.nl();
            }
            if (options['deps']) {
                this.env.log.msg('Installing dependencies may take several minutes.');
                this.env.log.rawmsg(yield getIonicDevAppText());
                const [installer, ...installerArgs] = yield pkgManagerArgs({ npmClient, shell: this.env.shell }, { command: 'install' });
                yield this.env.shell.run(installer, installerArgs, Object.assign({}, shellOptions, { stdio: 'inherit' }));
            }
            if (!clonedApp) {
                if (gitIntegration) {
                    yield this.env.shell.run('git', ['init'], shellOptions); // TODO: use initializeRepo()?
                }
                if (options['link'] && !linkConfirmed) {
                    this.env.log.rawmsg(yield getIonicProText());
                    const confirm = yield this.env.prompt({
                        type: 'confirm',
                        name: 'confirm',
                        message: 'Install the free Ionic Pro SDK and connect your app?',
                        noninteractiveValue: false,
                    });
                    if (confirm) {
                        linkConfirmed = true;
                    }
                }
                if (linkConfirmed) {
                    const [installer, ...installerArgs] = yield pkgManagerArgs({ npmClient, shell: this.env.shell }, { command: 'install', pkg: '@ionic/pro' });
                    yield this.env.shell.run(installer, installerArgs, shellOptions);
                    const cmdArgs = ['link'];
                    if (proAppId) {
                        cmdArgs.push(proAppId);
                    }
                    yield this.env.runCommand(cmdArgs);
                }
                const manifestPath = path.resolve(projectDir, 'ionic.starter.json');
                const manifest = yield this.loadManifest(manifestPath);
                if (manifest) {
                    yield fs_1.fsUnlink(manifestPath);
                }
                if (gitIntegration) {
                    yield this.env.shell.run('git', ['add', '-A'], shellOptions);
                    yield this.env.shell.run('git', ['commit', '-m', 'Initial commit', '--no-gpg-sign'], shellOptions);
                }
                if (manifest) {
                    yield this.performManifestOps(manifest);
                }
            }
            this.env.log.nl();
            yield this.showNextSteps(projectDir, linkConfirmed);
        });
    }
    isGitSetup() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const cmdInstalled = yield this.env.shell.cmdinfo('git', ['--version']);
            return Boolean(cmdInstalled);
        });
    }
    checkForExisting(projectName, projectDir) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const projectExists = yield fs_1.pathExists(projectName);
            if (projectExists) {
                const confirm = yield this.env.prompt({
                    type: 'confirm',
                    name: 'confirm',
                    message: (`The directory ${chalk_1.default.green(projectName)} contains file(s) that could conflict. ` +
                        'Would you like to overwrite the directory with this new project?'),
                    default: false,
                });
                if (!confirm) {
                    this.env.log.msg(`Not erasing existing project in ${chalk_1.default.green(format_1.prettyPath(projectDir))}.`);
                    throw new errors_1.FatalException();
                }
                this.canRemoveExisting = confirm;
            }
            this.env.tasks.end();
        });
    }
    findStarterTemplate(template, type, tag) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { STARTER_BASE_URL, STARTER_TEMPLATES, getStarterList } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/start'));
            const starterTemplate = STARTER_TEMPLATES.find(t => t.type === type && t.name === template);
            if (starterTemplate) {
                return Object.assign({}, starterTemplate, { archive: `${STARTER_BASE_URL}/${tag === 'latest' ? '' : `${tag}/`}${starterTemplate.id}.tar.gz` });
            }
            this.env.tasks.next('Looking up starter');
            const starterList = yield getStarterList(this.env.config, tag);
            const starter = starterList.starters.find(t => t.type === type && t.name === template);
            if (starter) {
                return Object.assign({}, starter, { archive: `${STARTER_BASE_URL}/${tag === 'latest' ? '' : `${tag}/`}${starter.id}.tar.gz` });
            }
            else {
                throw new errors_1.FatalException(`Unable to find starter template for ${chalk_1.default.green(template)}\n` +
                    `If this is not a typo, please make sure it is a valid starter template within the starters repo: ${chalk_1.default.bold('https://github.com/ionic-team/starters')}`);
            }
        });
    }
    validateName(name) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { isValidPackageName } = yield Promise.resolve().then(() => require('@ionic/cli-framework/utils/npm'));
            const { isProjectNameValid } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/start'));
            if (!isProjectNameValid(name)) {
                throw new errors_1.FatalException(`Please name your Ionic project something meaningful other than ${chalk_1.default.green(name)}`);
            }
            if (!isValidPackageName(name) || name !== path.basename(name)) {
                throw new errors_1.FatalException(`${chalk_1.default.green(name)} is not a valid package or directory name.\n` +
                    `Please choose a different name. Alphanumeric characters are always safe for app names. You can use the ${chalk_1.default.green('--display-name')} option for the human-friendly name.`);
            }
        });
    }
    loadManifest(manifestPath) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { readStarterManifest } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/start'));
            try {
                return yield readStarterManifest(manifestPath);
            }
            catch (e) {
                debug(`Error with manifest file ${chalk_1.default.bold(format_1.prettyPath(manifestPath))}: ${e}`);
            }
        });
    }
    performManifestOps(manifest) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (manifest.welcome) {
                this.env.log.nl();
                this.env.log.msg(`${chalk_1.default.bold('Starter Welcome')}:`);
                this.env.log.msg(manifest.welcome);
            }
        });
    }
    downloadStarterTemplate(projectDir, starterTemplate) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { download } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/http'));
            const { createTarExtraction } = yield Promise.resolve().then(() => require('@ionic/cli-utils/lib/utils/archive'));
            const task = this.env.tasks.next(`Downloading and extracting ${chalk_1.default.green(starterTemplate.name.toString())} starter`);
            const config = yield this.env.config.load();
            const ws = yield createTarExtraction({ cwd: projectDir, strip: starterTemplate.strip ? 1 : 0 });
            yield download(starterTemplate.archive, ws, {
                progress: (loaded, total) => task.progress(loaded, total),
                ssl: config.ssl,
            });
            this.env.tasks.end();
        });
    }
    showNextSteps(projectDir, linkConfirmed) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const steps = [
                `Go to your newly created project: ${chalk_1.default.green(`cd ${format_1.prettyPath(projectDir)}`)}`,
                `Get Ionic DevApp for easy device testing: ${chalk_1.default.bold('https://bit.ly/ionic-dev-app')}`,
            ];
            if (linkConfirmed) {
                steps.push(`Finish setting up Ionic Pro Error Monitoring: ${chalk_1.default.bold('https://ionicframework.com/docs/pro/monitoring/#getting-started')}`);
                steps.push(`Finally, push your code to Ionic Pro to perform real-time updates, and more: ${chalk_1.default.green('git push ionic master')}`);
            }
            this.env.log.info(`${chalk_1.default.bold('Next Steps')}:\n${steps.map(s => `- ${s}`).join('\n')}`);
        });
    }
}
exports.StartCommand = StartCommand;
